@@ -18,7 +18,7 @@ __all__ = (
 )
 
 __name__ = "conda"
-__version__ = "4.3.22"
+__version__ = "4.3.23"
 __author__ = "Continuum Analytics, Inc."
 __email__ = "conda@continuum.io"
 __license__ = "BSD"
@@ -87,6 +87,9 @@ class CondaMultiError(CondaError):
                     error="Multiple Errors Encountered.",
                     )
 
+    def contains(self, exception_class):
+        return any(isinstance(e, exception_class) for e in self.errors)
+
 
 class CondaExitZero(CondaError):
     pass
