@@ -1,15 +1,15 @@
-from __future__ import print_function, division, absolute_import
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 import errno
 import os
-import re
+from os.path import abspath, isdir
+import re as regex
 import sys
-from os.path import isdir, abspath
 
-from conda import text_type
-from ..exceptions import (CondaSystemExit, ArgumentError, CondaValueError, CondaEnvironmentError,
-                          TooManyArgumentsError, TooFewArgumentsError)
-from ..utils import on_win
+from ..common.compat import ensure_text_type, on_win, text_type
+from ..exceptions import (ArgumentError, CondaEnvironmentError, CondaSystemExit, CondaValueError,
+                          TooFewArgumentsError, TooManyArgumentsError)
+from ..utils import shells
 
 
 def help(command, shell):
@@ -42,15 +42,16 @@ of the environment ENV from the front of PATH.""")
 Removes the 'bin' directory of the environment activated with 'source
 activate' from PATH. """)
     else:
-        raise CondaSystemExit("No help available for command %s" % sys.argv[1])
+        raise CondaSystemExit("No help available for command %s" % ensure_text_type(sys.argv[1]))
 
 
-def prefix_from_arg(arg, shelldict):
-    from conda.base.context import context, locate_prefix_by_name
+def prefix_from_arg(arg, shell):
+    shelldict = shells[shell] if shell else {}
+    from ..base.context import context, locate_prefix_by_name
     'Returns a platform-native path'
     # MSYS2 converts Unix paths to Windows paths with unix seps
     # so we must check for the drive identifier too.
-    if shelldict['sep'] in arg and not re.match('[a-zA-Z]:', arg):
+    if shelldict['sep'] in arg and not regex.match('[a-zA-Z]:', arg):
         # strip is removing " marks, not \ - look carefully
         native_path = shelldict['path_from'](arg)
         if isdir(abspath(native_path.strip("\""))):
@@ -62,23 +63,23 @@ def prefix_from_arg(arg, shelldict):
     return prefix
 
 
-def binpath_from_arg(arg, shelldict):
-    # prefix comes back as platform-native path
-    prefix = prefix_from_arg(arg, shelldict=shelldict)
+def _get_prefix_paths(prefix):
     if on_win:
-        paths = [
-            prefix.rstrip("\\"),
-            os.path.join(prefix, 'Library', 'mingw-w64', 'bin'),
-            os.path.join(prefix, 'Library', 'usr', 'bin'),
-            os.path.join(prefix, 'Library', 'bin'),
-            os.path.join(prefix, 'Scripts'),
-                ]
+        yield prefix.rstrip("\\")
+        yield os.path.join(prefix, 'Library', 'mingw-w64', 'bin')
+        yield os.path.join(prefix, 'Library', 'usr', 'bin')
+        yield os.path.join(prefix, 'Library', 'bin')
+        yield os.path.join(prefix, 'Scripts')
     else:
-        paths = [
-            os.path.join(prefix, 'bin'),
-                ]
+        yield os.path.join(prefix, 'bin')
+
+
+def binpath_from_arg(arg, shell):
+    shelldict = shells[shell] if shell else {}
+    # prefix comes back as platform-native path
+    prefix = prefix_from_arg(arg, shell)
     # convert paths to shell-native paths
-    return [shelldict['path_to'](path) for path in paths]
+    return [shelldict['path_to'](path) for path in _get_prefix_paths(prefix)]
 
 
 def pathlist_to_str(paths, escape_backslashes=True):
@@ -89,74 +90,100 @@ def pathlist_to_str(paths, escape_backslashes=True):
     path = ' and '.join(paths)
     if on_win and escape_backslashes:
         # escape for printing to console - ends up as single \
-        path = re.sub(r'(?<!\\)\\(?!\\)', r'\\\\', path)
+        path = regex.sub(r'(?<!\\)\\(?!\\)', r'\\\\', path)
     else:
         path = path.replace("\\\\", "\\")
     return path
 
 
-def main():
-    from conda.base.context import context
-    from conda.base.constants import ROOT_ENV_NAME
-    from conda.utils import shells
-    if '-h' in sys.argv or '--help' in sys.argv:
-        # all execution paths sys.exit at end.
-        help(sys.argv[1], sys.argv[2])
+def get_activate_path(prefix, shell):
+    shelldict = shells[shell] if shell else {}
+    binpath = binpath_from_arg(prefix, shell)
 
-    if len(sys.argv) > 2:
-        shell = sys.argv[2]
-        shelldict = shells[shell]
-    if sys.argv[1] == '..activate':
-        arg_num = len(sys.argv)
+    # prepend our new entries onto the existing path and make sure that the separator is native
+    path = shelldict['pathsep'].join(binpath)
+    return path
+
+
+def main():
+    from ..base.constants import ROOT_ENV_NAME
+    from ..gateways.logging import initialize_logging
+    initialize_logging()
+
+    sys_argv = tuple(ensure_text_type(s) for s in sys.argv)
+
+    if '-h' in sys_argv or '--help' in sys_argv:
+        # all execution paths sys.exit at end.
+        help(sys_argv[1], sys_argv[2])
+
+    if len(sys_argv) > 2:
+        shell = sys_argv[2]
+    else:
+        shell = ''
+
+    if regex.match('^..(?:de|)activate$', sys_argv[1]):
+        arg_num = len(sys_argv)
         if arg_num != 4:
             num_expected = 2
             if arg_num < 4:
                 raise TooFewArgumentsError(num_expected, arg_num - num_expected,
-                                           "..activate expected exactly two arguments:\
-                                            shell and env name")
+                                           "{} expected exactly two arguments:\
+                                            shell and env name".format(sys_argv[1]))
             if arg_num > 4:
-                raise TooManyArgumentsError(num_expected, arg_num - num_expected, sys.argv[2:],
-                                            "..activate expected exactly two arguments:\
-                                             shell and env name")
-        binpath = binpath_from_arg(sys.argv[3], shelldict=shelldict)
+                raise TooManyArgumentsError(num_expected, arg_num - num_expected, sys_argv[2:],
+                                            "{} expected exactly two arguments:\
+                                             shell and env name".format(sys_argv[1]))
 
-        # prepend our new entries onto the existing path and make sure that the separator is native
-        path = shelldict['pathsep'].join(binpath)
+    if sys_argv[1] == '..activate':
+        print(get_activate_path(sys_argv[3], shell))
+        sys.exit(0)
 
-    # deactivation is handled completely in shell scripts - it restores backups of env variables.
-    #    It is done in shell scripts because they handle state much better than we can here.
+    elif sys_argv[1] == '..deactivate.path':
+        activation_path = get_activate_path(sys_argv[3], shell)
 
-    elif sys.argv[1] == '..checkenv':
-        if len(sys.argv) < 4:
+        if os.getenv('_CONDA_HOLD'):
+            new_path = regex.sub(r'%s(:?)' % regex.escape(activation_path),
+                                 r'CONDA_PATH_PLACEHOLDER\1',
+                                 os.environ[str('PATH')], 1)
+        else:
+            new_path = regex.sub(r'%s(:?)' % regex.escape(activation_path), r'',
+                                 os.environ[str('PATH')], 1)
+
+        print(new_path)
+        sys.exit(0)
+
+    elif sys_argv[1] == '..checkenv':
+        if len(sys_argv) < 4:
             raise ArgumentError("Invalid arguments to checkenv.  Need shell and env name/path")
-        if len(sys.argv) > 4:
+        if len(sys_argv) > 4:
             raise ArgumentError("did not expect more than one argument.")
-        if sys.argv[3].lower() == ROOT_ENV_NAME.lower():
+        if sys_argv[3].lower() == ROOT_ENV_NAME.lower():
             # no need to check root env and try to install a symlink there
             sys.exit(0)
             # raise CondaSystemExit
 
         # this should throw an error and exit if the env or path can't be found.
         try:
-            prefix = prefix_from_arg(sys.argv[3], shelldict=shelldict)
+            prefix = prefix_from_arg(sys_argv[3], shell)
         except ValueError as e:
             raise CondaValueError(text_type(e))
 
         # Make sure an env always has the conda symlink
         try:
-            import conda.install
-            conda.install.symlink_conda(prefix, context.root_dir, shell)
+            from ..base.context import context
+            from ..install import symlink_conda
+            symlink_conda(prefix, context.root_prefix, shell)
         except (IOError, OSError) as e:
             if e.errno == errno.EPERM or e.errno == errno.EACCES:
                 msg = ("Cannot activate environment {0}.\n"
                        "User does not have write access for conda symlinks."
-                       .format(sys.argv[2]))
+                       .format(sys_argv[2]))
                 raise CondaEnvironmentError(msg)
             raise
         sys.exit(0)
         # raise CondaSystemExit
-    elif sys.argv[1] == '..changeps1':
-        from conda.base.context import context
+    elif sys_argv[1] == '..changeps1':
+        from ..base.context import context
         path = int(context.changeps1)
 
     else:
