@@ -17,15 +17,27 @@ Limitations:
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 from abc import ABCMeta, abstractmethod
-from collections import Mapping, Set, defaultdict
-from enum import Enum
+from collections import Mapping, defaultdict
 from glob import glob
 from itertools import chain
 from logging import getLogger
 from os import environ, stat
-from os.path import join, basename
+from os.path import basename, join
 from stat import S_IFDIR, S_IFMT, S_IFREG
 
+from enum import Enum, EnumMeta
+
+from .compat import (isiterable, iteritems, itervalues, odict, primitive_types, string_types,
+                     text_type, with_metaclass)
+from .constants import EMPTY_MAP, NULL
+from .yaml import yaml_load
+from .. import CondaError, CondaMultiError
+from .._vendor.auxlib.collection import AttrDict, first, frozendict, last, make_immutable
+from .._vendor.auxlib.exceptions import ThisShouldNeverHappenError
+from .._vendor.auxlib.path import expand
+from .._vendor.auxlib.type_coercion import TypeCoercionError, typify_data_structure
+from .._vendor.boltons.setutils import IndexedSet
+
 try:
     from cytoolz.dicttoolz import merge
     from cytoolz.functoolz import excepts
@@ -41,19 +53,6 @@ except ImportError:  # pragma: no cover
     from ruamel.yaml.comments import CommentedSeq, CommentedMap  # pragma: no cover
     from ruamel.yaml.scanner import ScannerError
 
-from .. import CondaError, CondaMultiError
-from .._vendor.auxlib.collection import first, frozendict, last, AttrDict
-from .._vendor.auxlib.exceptions import ThisShouldNeverHappenError
-from .._vendor.auxlib.path import expand
-from .._vendor.auxlib.type_coercion import typify_data_structure, TypeCoercionError
-from ..base.constants import EMPTY_MAP, NULL
-from .compat import (isiterable, iteritems, odict, primitive_types, text_type,
-                     with_metaclass, string_types, itervalues)
-from .yaml import yaml_load
-
-__all__ = ["Configuration", "PrimitiveParameter",
-           "SequenceParameter", "MapParameter"]
-
 log = getLogger(__name__)
 
 
@@ -88,10 +87,6 @@ class ValidationError(ConfigurationError):
         self.source = source
         super(ConfigurationError, self).__init__(msg, **kwargs)
 
-        def __str__(self):
-            return ("Parameter %s = %r declared in %s is invalid."
-                    % (self.parameter_name, self.parameter_value, self.source))
-
 
 class MultipleKeysError(ValidationError):
 
@@ -110,7 +105,7 @@ class InvalidTypeError(ValidationError):
         self.valid_types = valid_types
         if msg is None:
             msg = ("Parameter %s = %r declared in %s has type %s.\n"
-                   "Valid types: %s." % (parameter_name, parameter_value,
+                   "Valid types:\n%s" % (parameter_name, parameter_value,
                                          source, wrong_type, pretty_list(valid_types)))
         super(InvalidTypeError, self).__init__(parameter_name, parameter_value, source, msg=msg)
 
@@ -150,7 +145,7 @@ def raise_errors(errors):
 
 
 class ParameterFlag(Enum):
-    final = 'final'
+    final = "final"
     top = "top"
     bottom = "bottom"
 
@@ -174,19 +169,6 @@ class ParameterFlag(Enum):
             return None
 
 
-# TODO: move elsewhere, probably auxlib
-# TODO: need to add order to at least frozendict, and preferrably frozenset
-def make_immutable(value):
-    if isinstance(value, Mapping):
-        return frozendict(value)
-    elif isinstance(value, Set):
-        return frozenset(value)
-    elif isiterable(value):
-        return tuple(value)
-    else:
-        return value
-
-
 @with_metaclass(ABCMeta)
 class RawParameter(object):
 
@@ -266,7 +248,7 @@ class ArgParseRawParameter(RawParameter):
         return None
 
     def valueflags(self, parameter_obj):
-        return None
+        return None if isinstance(parameter_obj, PrimitiveParameter) else ()
 
     @classmethod
     def make_raw_parameters(cls, args_from_argparse):
@@ -362,7 +344,8 @@ def load_file_configs(search_path):
         yield fullpath, YamlRawParameter.make_raw_parameters_from_file(fullpath)
 
     def _dir_yaml_loader(fullpath):
-        for filepath in glob(join(fullpath, "*.yml")):
+        for filepath in sorted(concatv(glob(join(fullpath, "*.yml")),
+                                       glob(join(fullpath, "*.yaml")))):
             yield filepath, YamlRawParameter.make_raw_parameters_from_file(filepath)
 
     # map a stat result to a file loader or a directory loader
@@ -523,7 +506,7 @@ class PrimitiveParameter(Parameter):
     python 2 has long and unicode types.
     """
 
-    def __init__(self, default, aliases=(), validation=None, parameter_type=None):
+    def __init__(self, default, aliases=(), validation=None, element_type=None):
         """
         Args:
             default (Any):  The parameter's default value.
@@ -531,11 +514,11 @@ class PrimitiveParameter(Parameter):
             validation (callable): Given a parameter value as input, return a boolean indicating
                 validity, or alternately return a string describing an invalid value. Returning
                 `None` also indicates a valid value.
-            parameter_type (type or Tuple[type]): Type-validation of parameter's value. If None,
+            element_type (type or Tuple[type]): Type-validation of parameter's value. If None,
                 type(default) is used.
 
         """
-        self._type = type(default) if parameter_type is None else parameter_type
+        self._type = type(default) if element_type is None else element_type
         self._element_type = self._type
         super(PrimitiveParameter, self).__init__(default, aliases, validation)
 
@@ -579,7 +562,6 @@ class SequenceParameter(Parameter):
 
     def collect_errors(self, instance, value, source="<<merged>>"):
         errors = super(SequenceParameter, self).collect_errors(instance, value)
-
         element_type = self._element_type
         for idx, element in enumerate(value):
             if not isinstance(element, element_type):
@@ -590,7 +572,12 @@ class SequenceParameter(Parameter):
     def _merge(self, matches):
         # get matches up to and including first important_match
         #   but if no important_match, then all matches are important_matches
-        relevant_matches = self._first_important_matches(matches)
+        relevant_matches_and_values = tuple((match, match.value(self)) for match in
+                                            self._first_important_matches(matches))
+        for match, value in relevant_matches_and_values:
+            if not isinstance(value, tuple):
+                raise InvalidTypeError(self.name, value, match.source, value.__class__.__name__,
+                                       self._type.__name__)
 
         # get individual lines from important_matches that were marked important
         # these will be prepended to the final result
@@ -598,17 +585,18 @@ class SequenceParameter(Parameter):
             return tuple(line
                          for line, flag in zip(match.value(parameter_obj),
                                                match.valueflags(parameter_obj))
-                         if flag is marker)
-        top_lines = concat(get_marked_lines(m, ParameterFlag.top, self) for m in relevant_matches)
+                         if flag is marker) if match else ()
+        top_lines = concat(get_marked_lines(m, ParameterFlag.top, self) for m, _ in
+                           relevant_matches_and_values)
 
         # also get lines that were marked as bottom, but reverse the match order so that lines
         # coming earlier will ultimately be last
-        bottom_lines = concat(get_marked_lines(m, ParameterFlag.bottom, self) for m in
-                              reversed(relevant_matches))
+        bottom_lines = concat(get_marked_lines(m, ParameterFlag.bottom, self) for m, _ in
+                              reversed(relevant_matches_and_values))
 
         # now, concat all lines, while reversing the matches
         #   reverse because elements closer to the end of search path take precedence
-        all_lines = concat(m.value(self) for m in reversed(relevant_matches))
+        all_lines = concat(v for _, v in reversed(relevant_matches_and_values))
 
         # stack top_lines + all_lines, then de-dupe
         top_deduped = tuple(unique(concatv(top_lines, all_lines)))
@@ -631,6 +619,14 @@ class SequenceParameter(Parameter):
                                        self._str_format_flag(valueflag)))
         return '\n'.join(lines)
 
+    def _get_all_matches(self, instance):
+        # this is necessary to handle argparse `action="append"`, which can't be set to a
+        #   default value of NULL
+        # it also config settings like `channels: ~`
+        matches, exceptions = super(SequenceParameter, self)._get_all_matches(instance)
+        matches = tuple(m for m in matches if m._raw_value is not None)
+        return matches, exceptions
+
 
 class MapParameter(Parameter):
     """Parameter type for a Configuration class that holds a map (i.e. dict) of python
@@ -658,23 +654,29 @@ class MapParameter(Parameter):
             errors.extend(InvalidElementTypeError(self.name, val, source, type(val),
                                                   element_type, key)
                           for key, val in iteritems(value) if not isinstance(val, element_type))
+
         return errors
 
     def _merge(self, matches):
         # get matches up to and including first important_match
         #   but if no important_match, then all matches are important_matches
-        relevant_matches = self._first_important_matches(matches)
+        relevant_matches_and_values = tuple((match, match.value(self)) for match in
+                                            self._first_important_matches(matches))
+        for match, value in relevant_matches_and_values:
+            if not isinstance(value, Mapping):
+                raise InvalidTypeError(self.name, value, match.source, value.__class__.__name__,
+                                       self._type.__name__)
 
         # mapkeys with important matches
         def key_is_important(match, key):
             return match.valueflags(self).get(key) is ParameterFlag.final
         important_maps = tuple(dict((k, v)
-                                    for k, v in iteritems(match.value(self))
+                                    for k, v in iteritems(match_value)
                                     if key_is_important(match, k))
-                               for match in relevant_matches)
+                               for match, match_value in relevant_matches_and_values)
         # dump all matches in a dict
         # then overwrite with important matches
-        return merge(concatv((m.value(self) for m in relevant_matches),
+        return merge(concatv((v for _, v in relevant_matches_and_values),
                              reversed(important_maps)))
 
     def repr_raw(self, raw_parameter):
@@ -687,6 +689,12 @@ class MapParameter(Parameter):
                                          self._str_format_flag(valueflag)))
         return '\n'.join(lines)
 
+    def _get_all_matches(self, instance):
+        # it also config settings like `proxy_servers: ~`
+        matches, exceptions = super(MapParameter, self)._get_all_matches(instance)
+        matches = tuple(m for m in matches if m._raw_value is not None)
+        return matches, exceptions
+
 
 class ConfigurationType(type):
     """metaclass for Configuration"""
@@ -705,35 +713,91 @@ class Configuration(object):
     def __init__(self, search_path=(), app_name=None, argparse_args=None):
         self.raw_data = odict()
         self._cache_ = dict()
+        self._reset_callbacks = set()  # TODO: make this a boltons ordered set
         self._validation_errors = defaultdict(list)
-        if search_path:
-            self._add_search_path(search_path)
-        if app_name is not None:
-            self._add_env_vars(app_name)
-        if argparse_args is not None:
-            self._add_argparse_args(argparse_args)
 
-    def _add_search_path(self, search_path):
-        return self._add_raw_data(load_file_configs(search_path))
+        if not hasattr(self, '_search_path') and search_path is not None:
+            # we only set search_path once; we never change it
+            self._search_path = IndexedSet(search_path)
 
-    def _add_env_vars(self, app_name):
-        self.raw_data[EnvRawParameter.source] = EnvRawParameter.make_raw_parameters(app_name)
-        self._cache_ = dict()
+        if not hasattr(self, '_app_name') and app_name is not None:
+            # we only set app_name once; we never change it
+            self._app_name = app_name
+
+        self._set_search_path(search_path)
+        self._set_env_vars(app_name)
+        self._set_argparse_args(argparse_args)
+
+    def _set_search_path(self, search_path):
+        if not hasattr(self, '_search_path') and search_path is not None:
+            # we only set search_path once; we never change it
+            self._search_path = IndexedSet(search_path)
+
+        if getattr(self, '_search_path', None):
+
+            # we need to make sure old data doesn't stick around if we are resetting
+            #   easiest solution is to completely clear raw_data and re-load other sources
+            #   if raw_data holds contents
+            raw_data_held_contents = bool(self.raw_data)
+            if raw_data_held_contents:
+                self.raw_data = odict()
+
+            self._set_raw_data(load_file_configs(search_path))
+
+            if raw_data_held_contents:
+                # this should only be triggered on re-initialization / reset
+                self._set_env_vars(getattr(self, '_app_name', None))
+                self._set_argparse_args(self._argparse_args)
+
+        self._reset_cache()
         return self
 
-    def _add_argparse_args(self, argparse_args):
-        self._argparse_args = AttrDict((k, v) for k, v, in iteritems(vars(argparse_args))
-                                       if v is not NULL)
+    def _set_env_vars(self, app_name=None):
+        if not hasattr(self, '_app_name') and app_name is not None:
+            # we only set app_name once; we never change it
+            self._app_name = app_name
+        if getattr(self, '_app_name', None):
+            erp = EnvRawParameter
+            self.raw_data[erp.source] = erp.make_raw_parameters(self._app_name)
+        self._reset_cache()
+        return self
+
+    def _set_argparse_args(self, argparse_args):
+        # the argparse_args we store internally in this class as self._argparse_args
+        #   will be a mapping type, not a non-`dict` object like argparse_args is natively
+        if hasattr(argparse_args, '__dict__'):
+            # the argparse_args from argparse will be an object with a __dict__ attribute
+            #   and not a mapping type like this method will turn it into
+            self._argparse_args = AttrDict((k, v) for k, v, in iteritems(vars(argparse_args))
+                                           if v is not NULL)
+        elif not argparse_args:
+            # argparse_args can be initialized as `None`
+            self._argparse_args = AttrDict()
+        else:
+            # we're calling this method with argparse_args that are a mapping type, likely
+            #   already having been processed by this method before
+            self._argparse_args = AttrDict((k, v) for k, v, in iteritems(argparse_args)
+                                           if v is not NULL)
+
         source = ArgParseRawParameter.source
         self.raw_data[source] = ArgParseRawParameter.make_raw_parameters(self._argparse_args)
-        self._cache_ = dict()
+        self._reset_cache()
         return self
 
-    def _add_raw_data(self, raw_data):
+    def _set_raw_data(self, raw_data):
         self.raw_data.update(raw_data)
-        self._cache_ = dict()
+        self._reset_cache()
         return self
 
+    def _reset_cache(self):
+        self._cache_ = dict()
+        for callback in self._reset_callbacks:
+            callback()
+        return self
+
+    def register_reset_callaback(self, callback):
+        self._reset_callbacks.add(callback)
+
     def check_source(self, source):
         # this method ends up duplicating much of the logic of Parameter.__get__
         # I haven't yet found a way to make it more DRY though
@@ -769,6 +833,24 @@ class Configuration(object):
         validation_errors = list(chain.from_iterable(self.check_source(source)[1]
                                                      for source in self.raw_data))
         raise_errors(validation_errors)
+        self.validate_configuration()
+
+    @staticmethod
+    def _collect_validation_error(func, *args, **kwargs):
+        try:
+            func(*args, **kwargs)
+        except ConfigurationError as e:
+            return e.errors if hasattr(e, 'errors') else e,
+        return ()
+
+    def validate_configuration(self):
+        errors = chain.from_iterable(Configuration._collect_validation_error(getattr, self, name)
+                                     for name in self.parameter_names)
+        post_errors = self.post_build_validation()
+        raise_errors(tuple(chain.from_iterable((errors, post_errors))))
+
+    def post_build_validation(self):
+        return ()
 
     def collect_all(self):
         typed_values = odict()
@@ -777,3 +859,49 @@ class Configuration(object):
             typed_values[source], validation_errors[source] = self.check_source(source)
         raise_errors(tuple(chain.from_iterable(itervalues(validation_errors))))
         return odict((k, v) for k, v in iteritems(typed_values) if v)
+
+    def describe_parameter(self, parameter_name):
+        # TODO, in Parameter base class, rename element_type to value_type
+        if parameter_name not in self.parameter_names:
+            parameter_name = '_' + parameter_name
+        parameter = self.__class__.__dict__[parameter_name]
+        assert isinstance(parameter, Parameter)
+
+        # dedupe leading underscore from name
+        name = parameter.name.lstrip('_')
+        aliases = tuple(alias for alias in parameter.aliases if alias != name)
+
+        description = self.get_descriptions()[name]
+        et = parameter._element_type
+        if type(et) == EnumMeta:
+            et = [et]
+        if not isiterable(et):
+            et = [et]
+        element_types = tuple(_et.__name__ for _et in et)
+
+        details = {
+            'parameter_type': parameter.__class__.__name__.lower().replace("parameter", ""),
+            'name': name,
+            'aliases': aliases,
+            'element_types': element_types,
+            'default_value': parameter.default,
+            'description': description.replace('\n', ' ').strip(),
+        }
+        if isinstance(parameter, SequenceParameter):
+            details['string_delimiter'] = parameter.string_delimiter
+        return details
+
+    def list_parameters(self):
+        return tuple(sorted(name.lstrip('_') for name in self.parameter_names))
+
+    def typify_parameter(self, parameter_name, value):
+        # return a tuple with correct parameter name and typed-value
+        if parameter_name not in self.parameter_names:
+            parameter_name = '_' + parameter_name
+        parameter = self.__class__.__dict__[parameter_name]
+        assert isinstance(parameter, Parameter)
+
+        return typify_data_structure(value, parameter._element_type)
+
+    def get_descriptions(self):
+        raise NotImplementedError()
