@@ -325,8 +325,7 @@ def write_pickled_repodata(cache_path, repodata):
         with open(get_pickle_path(cache_path), 'wb') as f:
             pickle.dump(repodata, f)
     except Exception as e:
-        import traceback
-        log.debug("Failed to dump pickled repodata.\n%s", traceback.format_exc())
+        log.debug("Failed to dump pickled repodata.", exc_info=True)
 
 
 def read_pickled_repodata(cache_path, channel_url, schannel, priority, etag, mod_stamp):
@@ -340,8 +339,7 @@ def read_pickled_repodata(cache_path, channel_url, schannel, priority, etag, mod
         with open(pickle_path, 'rb') as f:
             repodata = pickle.load(f)
     except Exception as e:
-        import traceback
-        log.debug("Failed to load pickled repodata.\n%s", traceback.format_exc())
+        log.debug("Failed to load pickled repodata.", exc_info=True)
         rm_rf(pickle_path)
         return None
 
