@@ -23,6 +23,17 @@ except ImportError:
 log = logging.getLogger(__name__)
 
 
+# TODO: for conda-build compatibility only
+# remove in conda 4.4
+class ResolvePackageNotFound(CondaError):  # change back to Exception in conda 4.4
+    def __init__(self, bad_deps):
+        # bad_deps is a list of lists
+        self.bad_deps = bad_deps
+        message = '\n' + '\n'.join(('  - %s' % dep) for deps in bad_deps for dep in deps if dep)
+        super(ResolvePackageNotFound, self).__init__(message)
+NoPackagesFound = NoPackagesFoundError = ResolvePackageNotFound  # NOQA
+
+
 class LockError(CondaError):
     def __init__(self, message):
         msg = "%s" % message
@@ -327,12 +338,7 @@ class MD5MismatchError(CondaError):
                                                actual_md5sum=actual_md5sum)
 
 
-class PackageNotFoundError(CondaError):
-    def __init__(self, message, **kwargs):
-        super(PackageNotFoundError, self).__init__(message, **kwargs)
-
-
-class PackageNotInstalledError(PackageNotFoundError):
+class PackageNotInstalledError(CondaError):
 
     def __init__(self, prefix, package_name):
         message = dals("""
@@ -355,6 +361,10 @@ class CondaHTTPError(CondaError):
         _message += "CF-RAY: %s\n\n" % cf_ray if cf_ray else "\n"
         message = _message + message
 
+        status_code = status_code or '000'
+        reason = reason or 'CONNECTION FAILED'
+        elapsed_time = elapsed_time or '-'
+
         from ._vendor.auxlib.logz import stringify
         response_details = (stringify(response) or '') if response else ''
 
@@ -379,33 +389,26 @@ class AuthenticationError(CondaError):
     pass
 
 
-class NoPackagesFoundError(CondaError):
-    """An exception to report that requested packages are missing.
+class PackageNotFoundError(CondaError):
 
-    Args:
-        bad_deps: a list of tuples of MatchSpecs, assumed to be dependency
-        chains, from top level to bottom.
-
-    Returns:
-        Raises an exception with a formatted message detailing the
-        missing packages and/or dependencies.
-    """
-
-    def __init__(self, bad_deps):
+    def __init__(self, bad_pkg, channel_urls=()):
         from .resolve import dashlist
-        from .base.context import context
+        channels = dashlist(channel_urls)
+
+        if not channel_urls:
+            msg = """Package(s) is missing from the environment:
+            %(pkg)s
+            """
 
-        deps = set(q[-1].spec for q in bad_deps)
-        if all(len(q) > 1 for q in bad_deps):
-            what = "Dependencies" if len(bad_deps) > 1 else "Dependency"
-        elif all(len(q) == 1 for q in bad_deps):
-            what = "Packages" if len(bad_deps) > 1 else "Package"
         else:
-            what = "Packages/dependencies"
-        bad_deps = dashlist(' -> '.join(map(str, q)) for q in bad_deps)
-        msg = '%s missing in current %s channels: %s' % (what, context.subdir, bad_deps)
-        super(NoPackagesFoundError, self).__init__(msg)
-        self.pkgs = deps
+            msg = """Packages missing in current channels:
+            %(pkg)s
+
+We have searched for the packages in the following channels:
+            %(channels)s
+            """
+
+        super(PackageNotFoundError, self).__init__(msg, pkg=bad_pkg, channels=channels)
 
 
 class UnsatisfiableError(CondaError):
@@ -440,7 +443,6 @@ class UnsatisfiableError(CondaError):
                         found = True
                 if not found:
                     chains[key] = [{val} for val in vals]
-            bad_deps = []
             for key, csets in iteritems(chains):
                 deps = []
                 for name, cset in zip(key, csets):
