@@ -4,11 +4,10 @@ from itertools import chain
 import logging
 import re
 
-from .base.constants import DEFAULTS_CHANNEL_NAME, MAX_CHANNEL_PRIORITY, CONDA_TARBALL_EXTENSION
+from .base.constants import CONDA_TARBALL_EXTENSION, DEFAULTS_CHANNEL_NAME, MAX_CHANNEL_PRIORITY
 from .base.context import context
 from .common.compat import iteritems, iterkeys, itervalues, string_types
-from .console import setup_handlers
-from .exceptions import CondaValueError, NoPackagesFoundError, UnsatisfiableError
+from .exceptions import CondaValueError, ResolvePackageNotFound, UnsatisfiableError
 from .logic import Clauses, minimal_unsatisfiable_subset
 from .models.dist import Dist
 from .models.index_record import IndexRecord
@@ -19,12 +18,11 @@ log = logging.getLogger(__name__)
 dotlog = logging.getLogger('dotupdate')
 stdoutlog = logging.getLogger('stdoutlog')
 stderrlog = logging.getLogger('stderrlog')
-setup_handlers()
-
 
 # used in conda build
 Unsatisfiable = UnsatisfiableError
-NoPackagesFound = NoPackagesFoundError
+ResolvePackageNotFound = ResolvePackageNotFound
+
 
 def dashlist(iter):
     return ''.join('\n  - ' + str(x) for x in iter)
@@ -299,7 +297,7 @@ class Resolve(object):
             filter = self.default_filter(feats)
             bad_deps.extend(self.invalid_chains(ms, filter))
         if bad_deps:
-            raise NoPackagesFoundError(bad_deps)
+            raise ResolvePackageNotFound(bad_deps)
         return spec2, feats
 
     def find_conflicts(self, specs):
@@ -562,7 +560,7 @@ class Resolve(object):
         ms = MatchSpec(ms)
         dists = self.find_matches(ms)
         if not dists and not emptyok:
-            raise NoPackagesFoundError([(ms,)])
+            raise ResolvePackageNotFound([(ms,)])
         return sorted(dists, key=self.version_key)
 
     @staticmethod
@@ -841,7 +839,8 @@ class Resolve(object):
     def solve(self, specs, returnall=False):
         # type: (List[str], bool) -> List[Dist]
         try:
-            stdoutlog.info("Solving package specifications: ")
+            if not context.json and not context.quiet:
+                stdoutlog.info("Solving package specifications: ")
             log.debug("Solving for %s", specs)
 
             # Find the compliant packages
@@ -953,7 +952,9 @@ class Resolve(object):
 
             def stripfeat(sol):
                 return sol.split('[')[0]
-            stdoutlog.info('\n')
+
+            if not context.quiet:
+                stdoutlog.info('\n')
 
             if returnall:
                 return [sorted(Dist(stripfeat(dname)) for dname in psol) for psol in psolutions]
